@@ -17,6 +17,8 @@
 #include <stdint.h>
 #include <string.h>  // memcmp
 
+#include <algorithm>  // std::fill
+
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "tests/mask_test.cc"
 #include "hwy/foreach_target.h"  // IWYU pragma: keep
@@ -189,7 +191,7 @@ HWY_NOINLINE void TestAllCountTrue() {
   ForAllTypes(ForPartialVectors<TestCountTrue>());
 }
 
-struct TestFindFirstTrue {
+struct TestFindFirstTrue {  // Also FindKnownFirstTrue
   template <class T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     using TI = MakeSigned<T>;  // For mask > 0 comparison
@@ -203,17 +205,18 @@ struct TestFindFirstTrue {
 
     HWY_ASSERT_EQ(intptr_t(-1), FindFirstTrue(d, MaskFalse(d)));
     HWY_ASSERT_EQ(intptr_t(0), FindFirstTrue(d, MaskTrue(d)));
+    HWY_ASSERT_EQ(size_t(0), FindKnownFirstTrue(d, MaskTrue(d)));
 
     for (size_t code = 1; code < (1ull << max_lanes); ++code) {
       for (size_t i = 0; i < max_lanes; ++i) {
         bool_lanes[i] = (code & (1ull << i)) ? TI(1) : TI(0);
       }
 
-      const intptr_t expected = static_cast<intptr_t>(
-          Num0BitsBelowLS1Bit_Nonzero32(static_cast<uint32_t>(code)));
+      const size_t expected =
+          Num0BitsBelowLS1Bit_Nonzero32(static_cast<uint32_t>(code));
       const auto mask = RebindMask(d, Gt(Load(di, bool_lanes.get()), Zero(di)));
-      const intptr_t actual = FindFirstTrue(d, mask);
-      HWY_ASSERT_EQ(expected, actual);
+      HWY_ASSERT_EQ(static_cast<intptr_t>(expected), FindFirstTrue(d, mask));
+      HWY_ASSERT_EQ(expected, FindKnownFirstTrue(d, mask));
     }
   }
 };
@@ -237,6 +240,11 @@ struct TestLogicalMask {
     HWY_ASSERT_MASK_EQ(d, m0, Not(m_all));
     HWY_ASSERT_MASK_EQ(d, m_all, Not(m0));
 
+    Print(d, ".", VecFromMask(d, ExclusiveNeither(m0, m0)));
+    HWY_ASSERT_MASK_EQ(d, m_all, ExclusiveNeither(m0, m0));
+    HWY_ASSERT_MASK_EQ(d, m0, ExclusiveNeither(m_all, m0));
+    HWY_ASSERT_MASK_EQ(d, m0, ExclusiveNeither(m0, m_all));
+
     // For all combinations of zero/nonzero state of subset of lanes:
     const size_t max_lanes = AdjustedLog2Reps(HWY_MIN(N, size_t(6)));
     for (size_t code = 0; code < (1ull << max_lanes); ++code) {
